#version 430 core

in vec3 worldPosition;
in vec3 worldNormal;
in vec3 directionToLight;
in vec3 directionToCamera;
in vec4 ShadowCoord;

// light parameters
uniform vec3 light_ambient;
uniform vec3 light_diffuse;

// material parameters
uniform vec3 material_color;
uniform vec3 material_ambient;
uniform vec3 material_diffuse;
uniform vec3 material_specular;
uniform float material_shininess;

uniform sampler2DShadow shadowMap;

layout(location = 0) out vec4 fragColor;

void main()
{

	float visibility = texture( shadowMap, vec3(ShadowCoord.xy, (ShadowCoord.z)/ShadowCoord.w) );

	vec3 ambient_term = material_ambient * light_ambient;
	fragColor.xyz = ambient_term.xyz;


	float distToLight = length(directionToLight);

	vec3 N = normalize(worldNormal);
	vec3 L = normalize(directionToLight);
	vec3 V = normalize(directionToCamera);
	
	float cosTheta = clamp(dot(N, L), 0, 1);

	if(cosTheta  > 0) {
		float constant_attenuation = 0.2;
		float linear_attenuation = 0.045;
		float quadratic_attenuation = 0.0075;
		float light_power = constant_attenuation + distToLight*linear_attenuation + distToLight*distToLight*quadratic_attenuation;

		vec3 light_intensity = light_diffuse / light_power;

		vec3 H = normalize(L + V);
		float hdotn = dot(H, N);

		//vec3 diffuse_term = (material_diffuse * light_diffuse * cosTheta).xyz;
		vec3 diffuse_term = visibility * ((material_diffuse) * light_intensity * cosTheta);
		vec3 specular_term = visibility * (pow(hdotn, material_shininess) * material_specular * light_intensity * cosTheta);

		fragColor.xyz += (diffuse_term + specular_term).xyz;
		}

	fragColor.xyz =  material_color * fragColor.xyz;
	fragColor.a = 1;
}