#version 430 core

in vec3 viewPosition;
in vec3 viewNormal;
in vec2 textureCoordinates;
in vec3 directionToLight;
in vec3 directionToCamera;
in vec3 coneDirection;
in vec4 smCoords;

//Texture
layout(binding=1) uniform sampler2D texture_diffuse1;

//shadow map
layout(binding=2) uniform sampler2DShadow shadowMap;

// light parameters
uniform vec3 light_ambient;
uniform vec3 light_diffuse;

// material parameters
uniform vec3 material_color;
uniform vec3 material_ambient;
uniform vec3 material_diffuse;
uniform vec3 material_specular;
uniform float material_shininess;

out vec4 fragColor;

//number of shades for toon shading
const float numOfShades = 4;

void main()
{

float shadowMult = textureProj(shadowMap, smCoords);
float attenuation = 1.0f;

vec4 textureColor = vec4(texture(texture_diffuse1, textureCoordinates));

	float distToLight = length(directionToLight);

	vec3 N = normalize(viewNormal);
	vec3 L = normalize(directionToLight);
	vec3 V = normalize(directionToCamera);
	
	float ambient_term = clamp(dot(material_ambient * light_ambient, vec3(1.0)), 0.0, 1.0);
	float specular_term = 0.0;
	float diffuse_term = 0.0;

	float cosTheta = clamp(dot(N, L), 0, 1);

	float outerConeAngle = 45;
	float innerConeAngle = 20;

	float lightToSurfaceAngle = degrees(acos(dot(-L, normalize(coneDirection))));
		if(lightToSurfaceAngle > outerConeAngle){
				attenuation = 0.0f;
		}
		else if(lightToSurfaceAngle < innerConeAngle) {
				attenuation = 1.0f;
		}
		else {
				attenuation = (outerConeAngle - lightToSurfaceAngle) / (outerConeAngle - innerConeAngle);
		}

	if(cosTheta  > 0) {
		float constant_attenuation = 0.2;
		float linear_attenuation = 0.045;
		float quadratic_attenuation = 0.0075;
		float light_power = constant_attenuation + distToLight*linear_attenuation + distToLight*distToLight*quadratic_attenuation;

		vec3 light_intensity = light_diffuse / light_power;

			diffuse_term = clamp(dot(attenuation * shadowMult * ((material_diffuse) * light_intensity * cosTheta), N), 0.0, 1.0);


			vec3 H = normalize(L + V);
			float hdotn = dot(H, N);

			float specular_term = clamp(dot(attenuation * shadowMult * (pow(clamp(hdotn, 0, 1), material_shininess) * material_specular * light_intensity * cosTheta), N), 0.0, 1.0);
		
		}

		//different vertices for debug
		//fragColor.xyz = (N.xyz + 1) / 2;
		//fragColor.xyz = (V.xyz + 1) / 2;
		//fragColor.xyz = (L.xyz + 1) / 2;

		float brightness = ambient_term + diffuse_term + specular_term;
		float shadeIntensity = ceil(brightness * numOfShades)/ numOfShades;

	fragColor.xyz = textureColor.xyz * shadeIntensity;
	fragColor.w = 1;
}