#version 430

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 texCoords;

out vec3 viewPosition;
out vec3 viewNormal;

out vec2 textureCoordinates;

out vec4 smCoords;

out vec3 directionToLight;
out vec3 directionToCamera;
out vec3 coneDirection;

uniform mat4 model;
uniform mat4 view;
uniform mat4 project;

uniform mat4 M_Vl_Pl_L_Matrix;

uniform vec3 lightPosition;
uniform vec3 cameraPosition;

void main(void) {
	
	viewPosition = (view * model * vec4(position,1)).xyz;

	mat4 normalMatrix = transpose(inverse(view * model)); // DO THIS ON CPU SIDE!
	viewNormal = (normalMatrix * vec4(normal,0)).xyz;

	textureCoordinates = texCoords;

	//Light Position
	vec3 L = (view * vec4(lightPosition, 1)).xyz;

	//Direction from Surface to Light
	directionToLight = (L - viewPosition).xyz;

	//Cone Direction
	coneDirection = (view * vec4(0, -1, 0, 0)).xyz;

	//FOR DIRECTIONAL LIGHT SHADOWS
	//directionToLight = vec3(0.5f, 2, 2);
	
	//Camera is always at (0/0/0) in Camera Space!
	//directionToCamera = normalize(cameraPosition - viewPosition.xyz);
	directionToCamera = normalize(vec3(0,0,0) - viewPosition.xyz);

	gl_Position = project * view * model * vec4(position, 1);

	smCoords = M_Vl_Pl_L_Matrix * vec4(position, 1);
}